#! /usr/bin/python
# coding=utf-8

# Copyright 2023 teapot
#
# Licensed under the Apache Licence, Version 2.0 (the "Licence");
# you may not use this file except in compliance with the Licence.
# You may obtain a copy of the Licence at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the Licence is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the Licence for the specific language governing permissions and
# limitations under the Licence.

# For more information on this plugin and example outcomes see:
#     http://gimpchat.com/viewtopic.php?f=9&t=20420

# Version 3.0.0

from gimpfu import *

PARENT_NONE = None
POSITION_TOP = 0

hue_filters = 'Hue saturation', 'Hue chroma'
HUE_SATURATION, HUE_CHROMA = range(2)

hue_chroma_message = '''Hue chroma requires gegl_command.py obtainable here:
http://gimpchat.com/viewtopic.php?f=9&t=18040&start=30p264354'''

def init():
    blurb = '''Make frames that can be used for an animation by repeatedly copying the input layer and changing the hue by 360 / number of frames each time.

''' + hue_chroma_message + '''

If the new layers are created in the existing image then the input layer is moved to its position in their sequence.
'''

    register(
        'plug-in-tp-rotate-hue',                   # Name
        blurb,                                     # Blurb
        blurb,                                     # Help
        'teapot',                                  # Author
        'teapot',                                  # Copyright holder
        '2023',                                    # Date
        'Rotate Hue...',                           # Menu name
        'RGB*,INDEXED*',                           # Image types
        [                                          # Parameters
            (PF_IMAGE, 'image', 'Input image', None),
            (PF_DRAWABLE, 'drawable', 'Input drawable', None),
            (PF_INT, 'frames', 'Number of frames', 18),
            (PF_OPTION, 'hue_filter', 'Hue filter', HUE_SATURATION, hue_filters),
            (PF_BOOL, 'reverse', 'Reverse order?', False),
            (PF_BOOL, 'new_image', 'Create new image?', False),
        ],
        [                                          # Results: [(type, name, description), ...]
        ],
        callback,                                  # Callback function
        menu='<Image>/Filters/Animation/Animators' # Menu path
    )

def callback(image, drawable, frames, hue_filter, reverse, new_image):
    if not pdb.gimp_item_is_layer(drawable):
        msg('Want a layer.')
        return

    if pdb.gimp_item_is_group(drawable):
        msg("Doesn't work on a layer group.")
        return

    if frames < 1:
        msg('Want a positive number of frames, got %s' % frames)
        return

    if hue_filter == HUE_CHROMA and not 'python_gegl' in dir(pdb):
        msg(hue_chroma_message)
        return

    if not new_image:
        pdb.gimp_image_undo_group_start(image)

    try:
        original_name = drawable.name

        if new_image:
            base_type = pdb.gimp_image_base_type(image)
            dest_image = pdb.gimp_image_new(drawable.width, drawable.height, base_type)
            layer = pdb.gimp_layer_new_from_drawable(drawable, dest_image)
            pdb.gimp_image_insert_layer(dest_image, layer, PARENT_NONE, POSITION_TOP)
            pdb.gimp_layer_set_offsets(layer, 0, 0)
        else:
            dest_image = image
            layer = drawable

        rotate_hue(dest_image, layer, original_name, frames, hue_filter, reverse)

        if new_image:
            display = pdb.gimp_display_new(dest_image)

    except Exception:
        import traceback
        msg(traceback.format_exc())

    finally:
        if not new_image:
            pdb.gimp_image_undo_group_end(image)

def rotate_hue(image, layer, original_name, frames, hue_filter, reverse):
    hue_offset = 0 # float -180 <= hue_offset <= 180
    lightness  = 0 # float -100 <= lightness  <= 100
    saturation = 0 # float -100 <= saturation <= 100
    overlap    = 0 # float    0 <= overlap    <= 100

    layer.name = '%s hue 0.00' % original_name
    step = 360.0 / frames

    for frame in range(1, frames):
        hue_offset = frame * step
        if hue_offset > 180:
            hue_offset -= 360
        if reverse:
            hue_offset = -hue_offset

        new_layer = layer.copy()
        new_layer.name = '%s hue %.2f' % (original_name, hue_offset)
        pdb.gimp_image_insert_layer(image, new_layer, PARENT_NONE, POSITION_TOP)

        if hue_filter == HUE_SATURATION:
            pdb.gimp_drawable_hue_saturation(new_layer, HUE_RANGE_ALL, hue_offset, lightness, saturation, overlap)

        elif hue_filter == HUE_CHROMA:
            gegl_string = 'hue-chroma hue=%f' % hue_offset
            pdb.python_gegl(image, new_layer, gegl_string)

        else:
            assert False

    if reverse:
        position = POSITION_TOP
    else:
        position = frames - 1
    pdb.gimp_image_reorder_item(image, layer, PARENT_NONE, position)

def msg(s):
    pdb.gimp_message('%s' % s)

init()
main()
